@@ -376,6 +376,8 @@ macro_rules! match_dtype_to_physical_apply_macro {
             DataType::Int64 => $macro!(i64 $(, $opt_args)*),
             #[cfg(feature = "dtype-i128")]
             DataType::Int128 => $macro!(i128 $(, $opt_args)*),
+            #[cfg(feature = "dtype-f16")]
+            DataType::Float16 => $macro!(pf16 $(, $opt_args)*),
             DataType::Float32 => $macro!(f32 $(, $opt_args)*),
             DataType::Float64 => $macro!(f64 $(, $opt_args)*),
             dt => panic!("not implemented for dtype {:?}", dt),
@@ -407,6 +409,8 @@ macro_rules! match_dtype_to_logical_apply_macro {
             DataType::Int64 => $macro!(Int64Type $(, $opt_args)*),
             #[cfg(feature = "dtype-i128")]
             DataType::Int128 => $macro!(Int128Type $(, $opt_args)*),
+            #[cfg(feature = "dtype-f16")]
+            DataType::Float16 => $macro!(Float16Type $(, $opt_args)*),
             DataType::Float32 => $macro!(Float32Type $(, $opt_args)*),
             DataType::Float64 => $macro!(Float64Type $(, $opt_args)*),
             dt => panic!("not implemented for dtype {:?}", dt),
@@ -437,6 +441,8 @@ macro_rules! match_arrow_dtype_apply_macro_ca {
             DataType::Int64 => $macro!($self.i64().unwrap() $(, $opt_args)*),
             #[cfg(feature = "dtype-i128")]
             DataType::Int128 => $macro!($self.i128().unwrap() $(, $opt_args)*),
+            #[cfg(feature = "dtype-f16")]
+            DataType::Float16 => $macro!($self.f16().unwrap() $(, $opt_args)*),
             DataType::Float32 => $macro!($self.f32().unwrap() $(, $opt_args)*),
             DataType::Float64 => $macro!($self.f64().unwrap() $(, $opt_args)*),
             dt => panic!("not implemented for dtype {:?}", dt),
@@ -449,6 +455,8 @@ macro_rules! with_match_physical_numeric_type {(
     $dtype:expr, | $_:tt $T:ident | $($body:tt)*
 ) => ({
     macro_rules! __with_ty__ {( $_ $T:ident ) => ( $($body)* )}
+    #[cfg(feature = "dtype-f16")]
+    use polars_utils::float16::pf16;
     use $crate::datatypes::DataType::*;
     match $dtype {
         #[cfg(feature = "dtype-i8")]
@@ -467,6 +475,8 @@ macro_rules! with_match_physical_numeric_type {(
         UInt64 => __with_ty__! { u64 },
         #[cfg(feature = "dtype-u128")]
         UInt128 => __with_ty__! { u128 },
+        #[cfg(feature = "dtype-f16")]
+        Float16 => __with_ty__! { pf16 },
         Float32 => __with_ty__! { f32 },
         Float64 => __with_ty__! { f64 },
         dt => panic!("not implemented for dtype {:?}", dt),
@@ -478,6 +488,8 @@ macro_rules! with_match_physical_integer_type {(
     $dtype:expr, | $_:tt $T:ident | $($body:tt)*
 ) => ({
     macro_rules! __with_ty__ {( $_ $T:ident ) => ( $($body)* )}
+    #[cfg(feature = "dtype-f16")]
+    use polars_utils::float16::pf16;
     use $crate::datatypes::DataType::*;
     match $dtype {
         #[cfg(feature = "dtype-i8")]
@@ -505,8 +517,11 @@ macro_rules! with_match_physical_float_type {(
     $dtype:expr, | $_:tt $T:ident | $($body:tt)*
 ) => ({
     macro_rules! __with_ty__ {( $_ $T:ident ) => ( $($body)* )}
+    use polars_utils::float16::pf16;
     use $crate::datatypes::DataType::*;
     match $dtype {
+        #[cfg(feature = "dtype-f16")]
+        Float16 => __with_ty__! { pf16 },
         Float32 => __with_ty__! { f32 },
         Float64 => __with_ty__! { f64 },
         dt => panic!("not implemented for dtype {:?}", dt),
@@ -520,6 +535,8 @@ macro_rules! with_match_physical_float_polars_type {(
     macro_rules! __with_ty__ {( $_ $T:ident ) => ( $($body)* )}
     use $crate::datatypes::DataType::*;
     match $key_type {
+        #[cfg(feature = "dtype-f16")]
+        Float16 => __with_ty__! { Float16Type },
         Float32 => __with_ty__! { Float32Type },
         Float64 => __with_ty__! { Float64Type },
         dt => panic!("not implemented for dtype {:?}", dt),
@@ -549,6 +566,8 @@ macro_rules! with_match_physical_numeric_polars_type {(
         UInt64 => __with_ty__! { UInt64Type },
             #[cfg(feature = "dtype-u128")]
         UInt128 => __with_ty__! { UInt128Type },
+            #[cfg(feature = "dtype-f16")]
+        Float16 => __with_ty__! { Float16Type },
         Float32 => __with_ty__! { Float32Type },
         Float64 => __with_ty__! { Float64Type },
         dt => panic!("not implemented for dtype {:?}", dt),
@@ -617,6 +636,8 @@ macro_rules! downcast_as_macro_arg_physical {
             DataType::Int64 => $macro!($self.i64().unwrap() $(, $opt_args)*),
             #[cfg(feature = "dtype-i128")]
             DataType::Int128 => $macro!($self.i128().unwrap() $(, $opt_args)*),
+            #[cfg(feature = "dtype-f16")]
+            DataType::Float16 => $macro!($self.f16().unwrap() $(, $opt_args)*),
             DataType::Float32 => $macro!($self.f32().unwrap() $(, $opt_args)*),
             DataType::Float64 => $macro!($self.f64().unwrap() $(, $opt_args)*),
             dt => panic!("not implemented for {:?}", dt),
@@ -677,6 +698,11 @@ macro_rules! downcast_as_macro_arg_physical_mut {
                 let ca: &mut Int128Chunked = $self.as_mut();
                 $macro!(Int128Type, ca $(, $opt_args)*)
             },
+            #[cfg(feature = "dtype-f16")]
+            DataType::Float16 => {
+                let ca: &mut Float16Chunked = $self.as_mut();
+                $macro!(Float16Type, ca $(, $opt_args)*)
+            },
             DataType::Float32 => {
                 let ca: &mut Float32Chunked = $self.as_mut();
                 $macro!(Float32Type, ca $(, $opt_args)*)
@@ -712,6 +738,8 @@ macro_rules! apply_method_all_arrow_series {
             DataType::Int64 => $self.i64().unwrap().$method($($args),*),
             #[cfg(feature = "dtype-i128")]
             DataType::Int128 => $self.i128().unwrap().$method($($args),*),
+            #[cfg(feature = "dtype-f16")]
+            DataType::Float16 => $self.f16().unwrap().$method($($args),*),
             DataType::Float32 => $self.f32().unwrap().$method($($args),*),
             DataType::Float64 => $self.f64().unwrap().$method($($args),*),
             DataType::Time => $self.time().unwrap().$method($($args),*),
@@ -754,6 +782,8 @@ macro_rules! apply_method_physical_integer {
 macro_rules! apply_method_physical_numeric {
     ($self:expr, $method:ident, $($args:expr),*) => {
         match $self.dtype() {
+            #[cfg(feature = "dtype-f16")]
+            DataType::Float16 => $self.f16().unwrap().$method($($args),*),
             DataType::Float32 => $self.f32().unwrap().$method($($args),*),
             DataType::Float64 => $self.f64().unwrap().$method($($args),*),
             _ => apply_method_physical_integer!($self, $method, $($args),*),
@@ -1233,20 +1263,40 @@ pub(crate) fn index_to_chunked_index_rev<
     )
 }
 
-pub fn first_non_null<'a, I>(iter: I) -> Option<usize>
+pub fn first_null<'a, I>(iter: I) -> Option<usize>
 where
-    I: Iterator<Item = Option<&'a Bitmap>>,
+    I: Iterator<Item = &'a dyn Array>,
 {
     let mut offset = 0;
-    for validity in iter {
-        if let Some(mask) = validity {
+    for arr in iter {
+        if let Some(mask) = arr.validity() {
+            let len_mask = mask.len();
+            let n = mask.leading_ones();
+            if n < len_mask {
+                return Some(offset + n);
+            }
+            offset += len_mask
+        } else {
+            offset += arr.len();
+        }
+    }
+    None
+}
+
+pub fn first_non_null<'a, I>(iter: I) -> Option<usize>
+where
+    I: Iterator<Item = &'a dyn Array>,
+{
+    let mut offset = 0;
+    for arr in iter {
+        if let Some(mask) = arr.validity() {
             let len_mask = mask.len();
             let n = mask.leading_zeros();
             if n < len_mask {
                 return Some(offset + n);
             }
             offset += len_mask
-        } else {
+        } else if !arr.is_empty() {
             return Some(offset);
         }
     }
@@ -1255,21 +1305,21 @@ where
 
 pub fn last_non_null<'a, I>(iter: I, len: usize) -> Option<usize>
 where
-    I: DoubleEndedIterator<Item = Option<&'a Bitmap>>,
+    I: DoubleEndedIterator<Item = &'a dyn Array>,
 {
     if len == 0 {
         return None;
     }
     let mut offset = 0;
-    for validity in iter.rev() {
-        if let Some(mask) = validity {
+    for arr in iter.rev() {
+        if let Some(mask) = arr.validity() {
             let len_mask = mask.len();
             let n = mask.trailing_zeros();
             if n < len_mask {
                 return Some(len - offset - n - 1);
             }
             offset += len_mask;
-        } else {
+        } else if !arr.is_empty() {
             return Some(len - offset - 1);
         }
     }
@@ -1320,22 +1370,6 @@ pub fn coalesce_nulls_columns(a: &Column, b: &Column) -> (Column, Column) {
     }
 }
 
-pub fn operation_exceeded_idxsize_msg(operation: &str) -> String {
-    if size_of::<IdxSize>() == size_of::<u32>() {
-        format!(
-            "{} exceeded the maximum supported limit of {} rows. Consider installing 'polars[rt64]'.",
-            operation,
-            IdxSize::MAX,
-        )
-    } else {
-        format!(
-            "{} exceeded the maximum supported limit of {} rows.",
-            operation,
-            IdxSize::MAX,
-        )
-    }
-}
-
 #[cfg(test)]
 mod test {
     use super::*;
