@@ -26,7 +26,7 @@ mod splitfields;
 mod utils;
 
 pub use options::{CommentPrefix, CsvEncoding, CsvParseOptions, CsvReadOptions, NullValues};
-pub use parser::{count_rows, count_rows_from_slice_par, count_rows_from_slice_raw};
+pub use parser::{count_rows, count_rows_from_slice_par};
 pub use read_impl::batched::{BatchedCsvReader, OwnedBatchedCsvReader};
 pub use reader::CsvReader;
 pub use schema_inference::infer_file_schema;
@@ -34,7 +34,7 @@ pub use schema_inference::infer_file_schema;
 pub mod _csv_read_internal {
     pub use super::buffer::validate_utf8;
     pub use super::options::NullValuesCompiled;
-    pub use super::parser::CountLines;
+    pub use super::parser::{CountLines, is_comment_line};
     pub use super::read_impl::{cast_columns, find_starting_point, read_chunk};
     pub use super::reader::prepare_csv_schema;
 }
