@@ -238,7 +238,10 @@ where
                     return Ok(BooleanChunked::full_null(ca_in.name().clone(), ca_in.len()));
                 }
 
-                let other = other.explode(true)?;
+                let other = other.explode(ExplodeOptions {
+                    empty_as_null: false,
+                    keep_nulls: true,
+                })?;
                 let other = other.as_ref().as_ref();
                 is_in_helper_ca(ca_in, other, nulls_equal)
             } else {
@@ -253,7 +256,10 @@ where
                     return Ok(BooleanChunked::full_null(ca_in.name().clone(), ca_in.len()));
                 }
 
-                let other = other.explode(true)?;
+                let other = other.explode(ExplodeOptions {
+                    empty_as_null: false,
+                    keep_nulls: true,
+                })?;
                 let other = other.as_ref().as_ref();
                 is_in_helper_ca(ca_in, other, nulls_equal)
             } else {
@@ -312,7 +318,10 @@ fn is_in_binary(
                     return Ok(BooleanChunked::full_null(ca_in.name().clone(), ca_in.len()));
                 }
 
-                let other = other.explode(true)?;
+                let other = other.explode(ExplodeOptions {
+                    empty_as_null: false,
+                    keep_nulls: true,
+                })?;
                 let other = other.binary()?;
                 is_in_helper_ca(ca_in, other, nulls_equal)
             } else {
@@ -327,7 +336,10 @@ fn is_in_binary(
                     return Ok(BooleanChunked::full_null(ca_in.name().clone(), ca_in.len()));
                 }
 
-                let other = other.explode(true)?;
+                let other = other.explode(ExplodeOptions {
+                    empty_as_null: false,
+                    keep_nulls: true,
+                })?;
                 let other = other.binary()?;
                 is_in_helper_ca(ca_in, other, nulls_equal)
             } else {
@@ -380,7 +392,10 @@ fn is_in_boolean(
                     return Ok(BooleanChunked::full_null(ca_in.name().clone(), ca_in.len()));
                 }
 
-                let other = other.explode(true)?;
+                let other = other.explode(ExplodeOptions {
+                    empty_as_null: false,
+                    keep_nulls: true,
+                })?;
                 let other = other.bool()?;
                 is_in_boolean_broadcast(ca_in, other, nulls_equal)
             } else {
@@ -395,7 +410,10 @@ fn is_in_boolean(
                     return Ok(BooleanChunked::full_null(ca_in.name().clone(), ca_in.len()));
                 }
 
-                let other = other.explode(true)?;
+                let other = other.explode(ExplodeOptions {
+                    empty_as_null: false,
+                    keep_nulls: true,
+                })?;
                 let other = other.bool()?;
                 is_in_boolean_broadcast(ca_in, other, nulls_equal)
             } else {
@@ -464,7 +482,10 @@ fn is_in_null(s: &Series, other: &Series, nulls_equal: bool) -> PolarsResult<Boo
                         return Ok(BooleanChunked::full_null(ca_in.name().clone(), ca_in.len()));
                     }
 
-                    let other = other.explode(true)?;
+                    let other = other.explode(ExplodeOptions {
+                        empty_as_null: false,
+                        keep_nulls: true,
+                    })?;
                     BooleanChunked::from_iter_values(
                         ca_in.name().clone(),
                         std::iter::repeat_n(other.has_nulls(), ca_in.len()),
@@ -483,7 +504,10 @@ fn is_in_null(s: &Series, other: &Series, nulls_equal: bool) -> PolarsResult<Boo
                         return Ok(BooleanChunked::full_null(ca_in.name().clone(), ca_in.len()));
                     }
 
-                    let other = other.explode(true)?;
+                    let other = other.explode(ExplodeOptions {
+                        empty_as_null: false,
+                        keep_nulls: true,
+                    })?;
                     BooleanChunked::from_iter_values(
                         ca_in.name().clone(),
                         std::iter::repeat_n(other.has_nulls(), ca_in.len()),
@@ -567,7 +591,10 @@ fn is_in_row_encoded(
                     return Ok(BooleanChunked::full_null(ca_in.name().clone(), ca_in.len()));
                 }
 
-                let other = other.explode(true)?;
+                let other = other.explode(ExplodeOptions {
+                    empty_as_null: false,
+                    keep_nulls: true,
+                })?;
                 let other = other.binary_offset()?;
                 is_in_helper_ca(&ca_in, other, nulls_equal)
             } else {
@@ -588,7 +615,10 @@ fn is_in_row_encoded(
                     return Ok(BooleanChunked::full_null(ca_in.name().clone(), ca_in.len()));
                 }
 
-                let other = other.explode(true)?;
+                let other = other.explode(ExplodeOptions {
+                    empty_as_null: false,
+                    keep_nulls: true,
+                })?;
                 let other = other.binary_offset()?;
                 is_in_helper_ca(&ca_in, other, nulls_equal)
             } else {
