@@ -9,6 +9,7 @@ use arrow::trusted_len::TrustMyLength;
 use arrow::types::NativeType;
 use polars_compute::min_max::MinMaxKernel;
 use polars_error::{PolarsResult, polars_bail};
+use polars_utils::float16::pf16;
 
 use super::binary::{
     build_statistics as binary_build_statistics, encode_plain as binary_encode_plain,
@@ -457,6 +458,7 @@ pub fn array_to_pages<K: DictionaryKey>(
                 ArrowDataType::UInt16 => dyn_prim!(u16, i32, array, options, type_),
                 ArrowDataType::UInt32 => dyn_prim!(u32, i32, array, options, type_),
                 ArrowDataType::UInt64 => dyn_prim!(u64, i64, array, options, type_),
+                ArrowDataType::Float16 => dyn_prim!(pf16, f32, array, options, type_),
                 ArrowDataType::Float32 => dyn_prim!(f32, f32, array, options, type_),
                 ArrowDataType::Float64 => dyn_prim!(f64, f64, array, options, type_),
                 ArrowDataType::LargeUtf8 => {
