@@ -1,4 +1,4 @@
-use std::fmt::{self, Display, Formatter};
+use std::fmt::{self, Display, Formatter, Write};
 
 use polars_core::frame::DataFrame;
 use polars_core::schema::Schema;
@@ -343,44 +343,50 @@ impl Display for ExprIRDisplay<'_> {
         use AExpr::*;
         match root {
             Element => f.write_str("element()"),
-            Window {
+            #[cfg(feature = "dynamic_group_by")]
+            Rolling {
+                function,
+                index_column,
+                period,
+                offset,
+                closed_window: _,
+            } => {
+                let function = self.with_root(function);
+                let index_column = self.with_root(index_column);
+                write!(
+                    f,
+                    "{function}.rolling(by='{index_column}', offset={offset}, period={period})",
+                )
+            },
+            Over {
                 function,
                 partition_by,
                 order_by,
-                options,
+                mapping: _,
             } => {
                 let function = self.with_root(function);
                 let partition_by = self.with_slice(partition_by);
-                match options {
-                    #[cfg(feature = "dynamic_group_by")]
-                    WindowType::Rolling(options) => {
-                        write!(
-                            f,
-                            "{function}.rolling(by='{}', offset={}, period={})",
-                            options.index_column, options.offset, options.period
-                        )
-                    },
-                    _ => {
-                        if let Some((order_by, _)) = order_by {
-                            let order_by = self.with_root(order_by);
-                            write!(
-                                f,
-                                "{function}.over(partition_by: {partition_by}, order_by: {order_by})"
-                            )
-                        } else {
-                            write!(f, "{function}.over({partition_by})")
-                        }
-                    },
+                if let Some((order_by, _)) = order_by {
+                    let order_by = self.with_root(order_by);
+                    write!(
+                        f,
+                        "{function}.over(partition_by: {partition_by}, order_by: {order_by})"
+                    )
+                } else {
+                    write!(f, "{function}.over({partition_by})")
                 }
             },
             Len => write!(f, "len()"),
-            Explode { expr, skip_empty } => {
+            Explode { expr, options } => {
                 let expr = self.with_root(expr);
-                if *skip_empty {
-                    write!(f, "{expr}.explode(skip_empty)")
-                } else {
-                    write!(f, "{expr}.explode()")
+                write!(f, "{expr}.explode(")?;
+                match (options.empty_as_null, options.keep_nulls) {
+                    (true, true) => {},
+                    (true, false) => f.write_str("keep_nulls=false")?,
+                    (false, true) => f.write_str("empty_as_null=false")?,
+                    (false, false) => f.write_str("empty_as_null=false, keep_nulls=false")?,
                 }
+                f.write_char(')')
             },
             Column(name) => write!(f, "col(\"{name}\")"),
             Literal(v) => write!(f, "{v:?}"),
@@ -455,7 +461,9 @@ impl Display for ExprIRDisplay<'_> {
                     Median(expr) => write!(f, "{}.median()", self.with_root(expr)),
                     Mean(expr) => write!(f, "{}.mean()", self.with_root(expr)),
                     First(expr) => write!(f, "{}.first()", self.with_root(expr)),
+                    FirstNonNull(expr) => write!(f, "{}.first_non_null()", self.with_root(expr)),
                     Last(expr) => write!(f, "{}.last()", self.with_root(expr)),
+                    LastNonNull(expr) => write!(f, "{}.last_non_null()", self.with_root(expr)),
                     Item { input, allow_empty } => {
                         self.with_root(input).fmt(f)?;
                         if *allow_empty {
@@ -524,7 +532,8 @@ impl Display for ExprIRDisplay<'_> {
                     write!(f, ".{function}()")
                 }
             },
-            AnonymousFunction { input, fmt_str, .. } => {
+            AnonymousFunction { input, fmt_str, .. }
+            | AnonymousStreamingAgg { input, fmt_str, .. } => {
                 let fst = self.with_root(&input[0]);
                 fst.fmt(f)?;
                 if input.len() >= 2 {
@@ -962,7 +971,7 @@ pub fn write_ir_non_recursive(
                 SinkTypeIR::Memory => "SINK (memory)",
                 SinkTypeIR::Callback { .. } => "SINK (callback)",
                 SinkTypeIR::File { .. } => "SINK (file)",
-                SinkTypeIR::Partition { .. } => "SINK (partition)",
+                SinkTypeIR::Partitioned { .. } => "SINK (partition)",
             };
             write!(f, "{:indent$}{name}", "")
         },
