@@ -146,11 +146,19 @@ impl PyExpr {
     fn unique_stable(&self) -> Self {
         self.inner.clone().unique_stable().into()
     }
-    fn first(&self) -> Self {
-        self.inner.clone().first().into()
+    fn first(&self, ignore_nulls: bool) -> Self {
+        if ignore_nulls {
+            self.inner.clone().first_non_null().into()
+        } else {
+            self.inner.clone().first().into()
+        }
     }
-    fn last(&self) -> Self {
-        self.inner.clone().last().into()
+    fn last(&self, ignore_nulls: bool) -> Self {
+        if ignore_nulls {
+            self.inner.clone().last_non_null().into()
+        } else {
+            self.inner.clone().last().into()
+        }
     }
     fn item(&self, allow_empty: bool) -> Self {
         self.inner.clone().item(allow_empty).into()
@@ -439,8 +447,14 @@ impl PyExpr {
         self.inner.clone().is_last_distinct().into()
     }
 
-    fn explode(&self) -> Self {
-        self.inner.clone().explode().into()
+    fn explode(&self, empty_as_null: bool, keep_nulls: bool) -> Self {
+        self.inner
+            .clone()
+            .explode(ExplodeOptions {
+                empty_as_null,
+                keep_nulls,
+            })
+            .into()
     }
 
     fn gather_every(&self, n: usize, offset: usize) -> Self {
@@ -618,19 +632,20 @@ impl PyExpr {
 
     fn rolling(
         &self,
-        index_column: &str,
+        index_column: PyExpr,
         period: &str,
         offset: &str,
         closed: Wrap<ClosedWindow>,
     ) -> PyResult<Self> {
-        let options = RollingGroupOptions {
-            index_column: index_column.into(),
-            period: Duration::try_parse(period).map_err(PyPolarsErr::from)?,
-            offset: Duration::try_parse(offset).map_err(PyPolarsErr::from)?,
-            closed_window: closed.0,
-        };
+        let period = Duration::try_parse(period).map_err(PyPolarsErr::from)?;
+        let offset = Duration::try_parse(offset).map_err(PyPolarsErr::from)?;
+        let closed = closed.0;
 
-        Ok(self.inner.clone().rolling(options).into())
+        Ok(self
+            .inner
+            .clone()
+            .rolling(index_column.inner, period, offset, closed)
+            .into())
     }
 
     fn and_(&self, expr: Self) -> Self {
@@ -701,8 +716,8 @@ impl PyExpr {
     fn reinterpret(&self, signed: bool) -> Self {
         self.inner.clone().reinterpret(signed).into()
     }
-    fn mode(&self) -> Self {
-        self.inner.clone().mode().into()
+    fn mode(&self, maintain_order: bool) -> Self {
+        self.inner.clone().mode(maintain_order).into()
     }
     fn interpolate(&self, method: Wrap<InterpolationMethod>) -> Self {
         self.inner.clone().interpolate(method.0).into()
