@@ -1,14 +1,18 @@
 use std::fmt::Write;
 
-use polars_plan::dsl::PartitionVariantIR;
+use polars_plan::dsl::{PartitionStrategyIR, PartitionVariantIR};
 use polars_plan::plans::expr_ir::ExprIR;
 use polars_plan::plans::{AExpr, EscapeLabel};
 use polars_plan::prelude::FileType;
+use polars_time::ClosedWindow;
+#[cfg(feature = "dynamic_group_by")]
+use polars_time::DynamicGroupOptions;
 use polars_utils::arena::Arena;
 use polars_utils::slice_enum::Slice;
 use slotmap::{Key, SecondaryMap, SlotMap};
 
 use super::{PhysNode, PhysNodeKey, PhysNodeKind};
+use crate::physical_plan::ZipBehavior;
 
 /// A style of a graph node.
 enum NodeStyle {
@@ -274,7 +278,7 @@ fn visualize_plan_rec(
             #[allow(unreachable_patterns)]
             _ => todo!(),
         },
-        PhysNodeKind::PartitionSink {
+        PhysNodeKind::PartitionedSink {
             input,
             file_type,
             variant,
@@ -299,6 +303,37 @@ fn visualize_plan_rec(
                 _ => todo!(),
             }
         },
+        PhysNodeKind::FileSink2 { input, options } => match options.file_format.as_ref() {
+            #[cfg(feature = "parquet")]
+            FileType::Parquet(_) => ("parquet-sink".to_string(), from_ref(input)),
+            #[cfg(feature = "ipc")]
+            FileType::Ipc(_) => ("ipc-sink".to_string(), from_ref(input)),
+            #[cfg(feature = "csv")]
+            FileType::Csv(_) => ("csv-sink".to_string(), from_ref(input)),
+            #[cfg(feature = "json")]
+            FileType::Json(_) => ("ndjson-sink".to_string(), from_ref(input)),
+            #[allow(unreachable_patterns)]
+            _ => todo!(),
+        },
+        PhysNodeKind::PartitionedSink2 { input, options } => {
+            let variant = match options.partition_strategy {
+                PartitionStrategyIR::Keyed { .. } => "partition-keyed",
+                PartitionStrategyIR::FileSize => "partition-file-size",
+            };
+
+            match options.file_format.as_ref() {
+                #[cfg(feature = "parquet")]
+                FileType::Parquet(_) => (format!("{variant}[parquet]"), from_ref(input)),
+                #[cfg(feature = "ipc")]
+                FileType::Ipc(_) => (format!("{variant}[ipc]"), from_ref(input)),
+                #[cfg(feature = "csv")]
+                FileType::Csv(_) => (format!("{variant}[csv]"), from_ref(input)),
+                #[cfg(feature = "json")]
+                FileType::Json(_) => (format!("{variant}[ndjson]"), from_ref(input)),
+                #[allow(unreachable_patterns)]
+                _ => todo!(),
+            }
+        },
         PhysNodeKind::InMemoryMap {
             input,
             map: _,
@@ -307,14 +342,50 @@ fn visualize_plan_rec(
             let mut label = String::new();
             label.push_str("in-memory-map");
             if let Some(format_str) = format_str {
-                label.push('\n');
+                label.write_str("\\n").unwrap();
 
                 let mut f = EscapeLabel(&mut label);
-                write!(f, "{format_str}").unwrap();
+                f.write_str(format_str).unwrap();
             }
             (label, from_ref(input))
         },
-        PhysNodeKind::Map { input, map: _ } => ("map".to_string(), from_ref(input)),
+        PhysNodeKind::Map {
+            input,
+            map: _,
+            format_str,
+        } => {
+            let mut label = String::new();
+            label.push_str("map");
+            if let Some(format_str) = format_str {
+                label.push_str("\\n");
+
+                let mut f = EscapeLabel(&mut label);
+                f.write_str(format_str).unwrap();
+            }
+            (label, from_ref(input))
+        },
+        PhysNodeKind::SortedGroupBy {
+            input,
+            key,
+            aggs,
+            slice,
+        } => {
+            let mut s = String::new();
+            s.push_str("sorted-group-by\\n");
+            let f = &mut s;
+            write!(f, "key: {key}\\n").unwrap();
+            if let Some((offset, length)) = slice {
+                write!(f, "slice: {offset}, {length}\\n").unwrap();
+            }
+            write!(
+                f,
+                "aggs:\\n{}",
+                fmt_exprs_to_label(aggs, expr_arena, FormatExprStyle::Select)
+            )
+            .unwrap();
+
+            (s, from_ref(input))
+        },
         PhysNodeKind::Sort {
             input,
             by_column,
@@ -379,12 +450,12 @@ fn visualize_plan_rec(
         PhysNodeKind::OrderedUnion { inputs } => ("ordered-union".to_string(), inputs.as_slice()),
         PhysNodeKind::Zip {
             inputs,
-            null_extend,
+            zip_behavior,
         } => {
-            let label = if *null_extend {
-                "zip-null-extend"
-            } else {
-                "zip"
+            let label = match zip_behavior {
+                ZipBehavior::NullExtend => "zip-null-extend",
+                ZipBehavior::Broadcast => "zip-broadcast",
+                ZipBehavior::Strict => "zip-strict",
             };
             (label.to_string(), inputs.as_slice())
         },
@@ -474,6 +545,94 @@ fn visualize_plan_rec(
             ),
             from_ref(input),
         ),
+        #[cfg(feature = "dynamic_group_by")]
+        PhysNodeKind::DynamicGroupBy {
+            input,
+            options,
+            aggs,
+            slice,
+        } => {
+            use polars_time::prelude::{Label, StartBy};
+
+            let DynamicGroupOptions {
+                index_column,
+                every,
+                period,
+                offset,
+                label,
+                include_boundaries,
+                closed_window,
+                start_by,
+            } = options;
+            let mut s = String::new();
+            let f = &mut s;
+            f.write_str("dynamic-group-by\\n").unwrap();
+            write!(f, "index column: {index_column}\\n").unwrap();
+            write!(f, "every: {every}").unwrap();
+            if every != period {
+                write!(f, ", period: {period}").unwrap();
+            }
+            if !offset.is_zero() {
+                write!(f, ", offset: {offset}").unwrap();
+            }
+            f.write_str("\\n").unwrap();
+            if *label != Label::Left {
+                write!(f, "label: {}\\n", <&'static str>::from(label)).unwrap();
+            }
+            if *include_boundaries {
+                write!(f, "include_boundaries: true\\n").unwrap();
+            }
+            if *start_by != StartBy::WindowBound {
+                write!(f, "start_by: {}\\n", <&'static str>::from(start_by)).unwrap();
+            }
+            if *closed_window != ClosedWindow::Left {
+                write!(
+                    f,
+                    "closed_window: {}\\n",
+                    <&'static str>::from(closed_window)
+                )
+                .unwrap();
+            }
+            if let Some((offset, length)) = slice {
+                write!(f, "slice: {offset}, {length}\\n").unwrap();
+            }
+            write!(
+                f,
+                "aggs:\\n{}",
+                fmt_exprs_to_label(aggs, expr_arena, FormatExprStyle::Select)
+            )
+            .unwrap();
+
+            (s, from_ref(input))
+        },
+        #[cfg(feature = "dynamic_group_by")]
+        PhysNodeKind::RollingGroupBy {
+            input,
+            index_column,
+            period,
+            offset,
+            closed,
+            slice,
+            aggs,
+        } => {
+            let mut s = String::new();
+            let f = &mut s;
+            f.write_str("rolling-group-by\\n").unwrap();
+            write!(f, "index column: {index_column}\\n").unwrap();
+            write!(f, "period: {period}, offset: {offset}\\n").unwrap();
+            write!(f, "closed: {}\\n", <&'static str>::from(*closed)).unwrap();
+            if let Some((offset, length)) = slice {
+                write!(f, "slice: {offset}, {length}\\n").unwrap();
+            }
+            write!(
+                f,
+                "aggs:\\n{}",
+                fmt_exprs_to_label(aggs, expr_arena, FormatExprStyle::Select)
+            )
+            .unwrap();
+
+            (s, from_ref(input))
+        },
         PhysNodeKind::InMemoryJoin {
             input_left,
             input_right,
@@ -553,6 +712,10 @@ fn visualize_plan_rec(
         } => ("merge-sorted".to_string(), &[*input_left, *input_right][..]),
         #[cfg(feature = "ewma")]
         PhysNodeKind::EwmMean { input, options: _ } => ("ewm-mean".to_string(), &[*input][..]),
+        #[cfg(feature = "ewma")]
+        PhysNodeKind::EwmVar { input, options: _ } => ("ewm-var".to_string(), &[*input][..]),
+        #[cfg(feature = "ewma")]
+        PhysNodeKind::EwmStd { input, options: _ } => ("ewm-std".to_string(), &[*input][..]),
     };
 
     let node_id = node_key.data().as_ffi();
