@@ -53,14 +53,14 @@ def test_arrow_empty_dataframe() -> None:
     df = pl.DataFrame({})
     tbl = pa.table({})
     assert df.to_arrow() == tbl
-    df2 = cast(pl.DataFrame, pl.from_arrow(df.to_arrow()))
+    df2 = cast("pl.DataFrame", pl.from_arrow(df.to_arrow()))
     assert_frame_equal(df2, df)
 
     # 0 row dataframe
     df = pl.DataFrame({}, schema={"a": pl.Int32})
     tbl = pa.Table.from_batches([], pa.schema([pa.field("a", pa.int32())]))
     assert df.to_arrow() == tbl
-    df2 = cast(pl.DataFrame, pl.from_arrow(df.to_arrow()))
+    df2 = cast("pl.DataFrame", pl.from_arrow(df.to_arrow()))
     assert df2.schema == {"a": pl.Int32}
     assert df2.shape == (0, 1)
 
@@ -81,7 +81,7 @@ def test_arrow_dict_to_polars() -> None:
 def test_arrow_list_chunked_array() -> None:
     a = pa.array([[1, 2], [3, 4]])
     ca = pa.chunked_array([a, a, a])
-    s = cast(pl.Series, pl.from_arrow(ca))
+    s = cast("pl.Series", pl.from_arrow(ca))
     assert s.dtype == pl.List
 
 
@@ -300,7 +300,7 @@ def test_upcast_pyarrow_dicts() -> None:
     ]
 
     tbl = pa.concat_tables(tbls, promote_options="default")
-    out = cast(pl.DataFrame, pl.from_arrow(tbl))
+    out = cast("pl.DataFrame", pl.from_arrow(tbl))
     assert out.shape == (128, 1)
     assert out["col_name"][0] == "value_0"
     assert out["col_name"][127] == "value_127"
@@ -315,25 +315,25 @@ def test_no_rechunk() -> None:
 
 
 def test_from_empty_arrow() -> None:
-    df = cast(pl.DataFrame, pl.from_arrow(pa.table(pd.DataFrame({"a": [], "b": []}))))
+    df = cast("pl.DataFrame", pl.from_arrow(pa.table(pd.DataFrame({"a": [], "b": []}))))
     assert df.columns == ["a", "b"]
     assert df.dtypes == [pl.Float64, pl.Float64]
 
     # 2705
     df1 = pd.DataFrame(columns=["b"], dtype=float, index=pd.Index([]))
     tbl = pa.Table.from_pandas(df1)
-    out = cast(pl.DataFrame, pl.from_arrow(tbl))
+    out = cast("pl.DataFrame", pl.from_arrow(tbl))
     assert out.columns == ["b", "__index_level_0__"]
     assert out.dtypes == [pl.Float64, pl.Null]
     tbl = pa.Table.from_pandas(df1, preserve_index=False)
-    out = cast(pl.DataFrame, pl.from_arrow(tbl))
+    out = cast("pl.DataFrame", pl.from_arrow(tbl))
     assert out.columns == ["b"]
     assert out.dtypes == [pl.Float64]
 
     # 4568
     tbl = pa.table({"l": []}, schema=pa.schema([("l", pa.large_list(pa.uint8()))]))
 
-    df = cast(pl.DataFrame, pl.from_arrow(tbl))
+    df = cast("pl.DataFrame", pl.from_arrow(tbl))
     assert df.schema["l"] == pl.List(pl.UInt8)
 
 
@@ -350,7 +350,7 @@ def test_cat_int_types_3500() -> None:
     uint_dict_type = pa.dictionary(index_type=pa.uint8(), value_type=pa.utf8())
 
     for t in [int_dict_type, uint_dict_type]:
-        s = cast(pl.Series, pl.from_arrow(pyarrow_array.cast(t)))
+        s = cast("pl.Series", pl.from_arrow(pyarrow_array.cast(t)))
         assert_series_equal(
             s, pl.Series(["a", "a", "b"]).cast(pl.Categorical), check_names=False
         )
@@ -396,7 +396,7 @@ def test_from_pyarrow_map() -> None:
 def test_from_fixed_size_binary_list() -> None:
     val = [[b"63A0B1C66575DD5708E1EB2B"]]
     arrow_array = pa.array(val, type=pa.list_(pa.binary(24)))
-    s = cast(pl.Series, pl.from_arrow(arrow_array))
+    s = cast("pl.Series", pl.from_arrow(arrow_array))
     assert s.dtype == pl.List(pl.Binary)
     assert s.to_list() == val
 
@@ -407,7 +407,7 @@ def test_dataframe_from_repr() -> None:
         pl.LazyFrame(
             {
                 "a": [1, 2, None],
-                "b": [4.5, 5.5, 6.5],
+                "b": [4.5, 5.23e13, -3.12e12],
                 "c": ["x", "y", "z"],
                 "d": [True, False, True],
                 "e": [None, "", None],
@@ -430,19 +430,19 @@ def test_dataframe_from_repr() -> None:
     assert frame.schema == {
         "a": pl.Int64,
         "b": pl.Float64,
-        "c": pl.Categorical(ordering="lexical"),
+        "c": pl.Categorical(),
         "d": pl.Boolean,
         "e": pl.String,
         "f": pl.Date,
         "g": pl.Time,
         "h": pl.Datetime("ns"),
     }
-    df = cast(pl.DataFrame, pl.from_repr(repr(frame)))
+    df = cast("pl.DataFrame", pl.from_repr(repr(frame)))
     assert_frame_equal(frame, df)
 
     # empty frame; confirm schema is inferred
     df = cast(
-        pl.DataFrame,
+        "pl.DataFrame",
         pl.from_repr(
             """
             ┌─────┬─────┬─────┬─────┬─────┬───────┐
@@ -467,7 +467,7 @@ def test_dataframe_from_repr() -> None:
 
     # empty frame with no dtypes
     df = cast(
-        pl.DataFrame,
+        "pl.DataFrame",
         pl.from_repr(
             """
             ┌──────┬───────┐
@@ -481,7 +481,7 @@ def test_dataframe_from_repr() -> None:
 
     # empty frame with a non-standard/blank 'null' in numeric col
     df = cast(
-        pl.DataFrame,
+        "pl.DataFrame",
         pl.from_repr(
             """
             ┌─────┬──────┐
@@ -504,7 +504,7 @@ def test_dataframe_from_repr() -> None:
     )
 
     df = cast(
-        pl.DataFrame,
+        "pl.DataFrame",
         pl.from_repr(
             """
             # >>> Missing cols with old-style ellipsis, nulls, commented out
@@ -537,7 +537,7 @@ def test_dataframe_from_repr() -> None:
     ]
 
     df = cast(
-        pl.DataFrame,
+        "pl.DataFrame",
         pl.from_repr(
             """
             # >>> no dtypes:
@@ -559,7 +559,7 @@ def test_dataframe_from_repr() -> None:
     ]
 
     df = cast(
-        pl.DataFrame,
+        "pl.DataFrame",
         pl.from_repr(
             """
             In [2]: with pl.Config() as cfg:
@@ -594,7 +594,7 @@ def test_dataframe_from_repr() -> None:
 
 def test_dataframe_from_repr_24110() -> None:
     df = cast(
-        pl.DataFrame,
+        "pl.DataFrame",
         pl.from_repr("""
             shape: (7, 1)
             ┌──────────────┐
@@ -625,7 +625,7 @@ def test_dataframe_from_repr_24110() -> None:
 
 def test_dataframe_from_duckdb_repr() -> None:
     df = cast(
-        pl.DataFrame,
+        "pl.DataFrame",
         pl.from_repr(
             """
             # misc streaming stats
@@ -685,11 +685,11 @@ def test_series_from_repr() -> None:
     )
 
     for col in frame.columns:
-        s = cast(pl.Series, pl.from_repr(repr(frame[col])))
+        s = cast("pl.Series", pl.from_repr(repr(frame[col])))
         assert_series_equal(s, frame[col])
 
     s = cast(
-        pl.Series,
+        "pl.Series",
         pl.from_repr(
             """
             Out[3]:
@@ -706,7 +706,7 @@ def test_series_from_repr() -> None:
     assert_series_equal(s, pl.Series("s", ["a", "c"]))
 
     s = cast(
-        pl.Series,
+        "pl.Series",
         pl.from_repr(
             """
             Series: 'flt' [f32]
@@ -718,7 +718,7 @@ def test_series_from_repr() -> None:
     assert_series_equal(s, pl.Series("flt", [], dtype=pl.Float32))
 
     s = cast(
-        pl.Series,
+        "pl.Series",
         pl.from_repr(
             """
             Series: 'flt' [f64]
@@ -749,7 +749,7 @@ def test_dataframe_from_repr_custom_separators() -> None:
     # repr created with custom digit-grouping
     # and non-default group/decimal separators
     df = cast(
-        pl.DataFrame,
+        "pl.DataFrame",
         pl.from_repr(
             """
             ┌───────────┬────────────┐
@@ -793,12 +793,12 @@ def test_sliced_struct_from_arrow() -> None:
 
     # slice the table
     # check if FFI correctly reads sliced
-    result = cast(pl.DataFrame, pl.from_arrow(tbl.slice(1, 2)))
+    result = cast("pl.DataFrame", pl.from_arrow(tbl.slice(1, 2)))
     assert result.to_dict(as_series=False) == {
         "struct_col": [{"a": 2, "b": "bar"}, {"a": 3, "b": "baz"}]
     }
 
-    result = cast(pl.DataFrame, pl.from_arrow(tbl.slice(1, 1)))
+    result = cast("pl.DataFrame", pl.from_arrow(tbl.slice(1, 1)))
     assert result.to_dict(as_series=False) == {"struct_col": [{"a": 2, "b": "bar"}]}
 
 
@@ -829,7 +829,7 @@ def test_from_arrow_fixed_offset(fixed_offset: str, etc_tz: str) -> None:
         [datetime(2021, 1, 1, 0, 0, 0, 0)],
         type=pa.timestamp("us", tz=fixed_offset),
     )
-    result = cast(pl.Series, pl.from_arrow(arr))
+    result = cast("pl.Series", pl.from_arrow(arr))
     expected = pl.Series(
         [datetime(2021, 1, 1, tzinfo=timezone.utc)]
     ).dt.convert_time_zone(etc_tz)
@@ -840,7 +840,7 @@ def test_from_avro_valid_time_zone_13032() -> None:
     arr = pa.array(
         [datetime(2021, 1, 1, 0, 0, 0, 0)], type=pa.timestamp("ns", tz="00:00")
     )
-    result = cast(pl.Series, pl.from_arrow(arr))
+    result = cast("pl.Series", pl.from_arrow(arr))
     expected = pl.Series([datetime(2021, 1, 1)], dtype=pl.Datetime("ns", "UTC"))
     assert_series_equal(result, expected)
 
@@ -868,6 +868,7 @@ def test_compat_level(monkeypatch: pytest.MonkeyPatch) -> None:
     assert oldest._version == 0
     with pytest.warns(UnstableWarning):
         newest = CompatLevel.newest()
+    with pytest.warns(UnstableWarning):
         assert newest is CompatLevel.newest()
     assert newest._version == 1
 
@@ -897,13 +898,6 @@ def test_compat_level(monkeypatch: pytest.MonkeyPatch) -> None:
         df.to_arrow(compat_level=newest)["bin_col"][0], pa.BinaryViewScalar
     )
 
-    assert len(df.write_ipc(None).getbuffer()) == 738
-    assert len(df.write_ipc(None, compat_level=oldest).getbuffer()) == 866
-    assert len(df.write_ipc(None, compat_level=newest).getbuffer()) == 738
-    assert len(df.write_ipc_stream(None).getbuffer()) == 520
-    assert len(df.write_ipc_stream(None, compat_level=oldest).getbuffer()) == 648
-    assert len(df.write_ipc_stream(None, compat_level=newest).getbuffer()) == 520
-
 
 def test_df_pycapsule_interface() -> None:
     df = pl.DataFrame(
@@ -941,11 +935,7 @@ def test_misaligned_nested_arrow_19097() -> None:
 
 
 def test_arrow_roundtrip_lex_cat_20288() -> None:
-    tb = (
-        pl.Series("a", ["A", "B"], pl.Categorical(ordering="lexical"))
-        .to_frame()
-        .to_arrow()
-    )
+    tb = pl.Series("a", ["A", "B"], pl.Categorical()).to_frame().to_arrow()
     df = pl.from_arrow(tb)
     assert isinstance(df, pl.DataFrame)
     dt = df.schema["a"]
@@ -1072,7 +1062,7 @@ def test_schema_constructor_from_schema_capsule() -> None:
 
     with pytest.raises(
         ValueError,
-        match="object passed to pl.Schema did not return struct dtype: object: pyarrow.Field<a: int32>, dtype: Int32",
+        match=r"object passed to pl.Schema did not return struct dtype: object: pyarrow\.Field<a: int32>, dtype: Int32",
     ):
         pl.Schema(pa.field("a", pa.int32()))
 
@@ -1083,7 +1073,7 @@ def test_schema_constructor_from_schema_capsule() -> None:
 
     with pytest.raises(
         DuplicateError,
-        match="iterable passed to pl.Schema contained duplicate name 'a'",
+        match=r"iterable passed to pl\.Schema contained duplicate name 'a'",
     ):
         pl.Schema([pa.field("a", pa.int32()), pa.field("a", pa.int64())])
 
@@ -1395,3 +1385,13 @@ def test_month_day_nano_from_ffi_15969(monkeypatch: pytest.MonkeyPatch) -> None:
     # TODO: Add Parquet round-trip test if this starts working.
     with pytest.raises(pa.ArrowNotImplementedError):
         pq.write_table(arrow_tbl, f)
+
+
+def test_schema_to_arrow_15563() -> None:
+    assert pl.Schema({"x": pl.String}).to_arrow() == pa.schema(
+        [pa.field("x", pa.string_view())]
+    )
+
+    assert pl.Schema({"x": pl.String}).to_arrow(
+        compat_level=CompatLevel.oldest()
+    ) == pa.schema([pa.field("x", pa.large_string())])
