@@ -21,7 +21,7 @@ def foods_ipc_path() -> Path:
 
 
 def test_any_all() -> None:
-    df = pl.DataFrame(  # noqa: F841
+    df = pl.DataFrame(
         {
             "x": [-1, 0, 1, 2, 3, 4],
             "y": [1, 0, 0, 1, 2, 3],
@@ -135,15 +135,15 @@ def test_count() -> None:
 
 
 def test_cte_aliasing() -> None:
-    df1 = pl.DataFrame({"colx": ["aa", "bb"], "coly": [40, 30]})  # noqa: F841
-    df2 = pl.DataFrame({"colx": "aa", "colz": 20})  # noqa: F841
+    df1 = pl.DataFrame({"colx": ["aa", "bb"], "coly": [40, 30]})
+    df2 = pl.DataFrame({"colx": "aa", "colz": 20})
     df3 = pl.sql(
         query="""
             WITH
               test1 AS (SELECT * FROM df1),
               test2 AS (SELECT * FROM df2),
               test3 AS (
-                SELECT t1.colx, t2.colz
+                SELECT ROW_NUMBER() OVER (ORDER BY colx) AS n, t1.colx, t2.colz
                 FROM test1 t1
                 LEFT JOIN test2 t2 ON t1.colx = t2.colx
             )
@@ -151,7 +151,8 @@ def test_cte_aliasing() -> None:
         """,
         eager=True,
     )
-    assert df3.rows() == [("bb", None), ("aa", 20)]
+    expected = [(2, "bb", None), (1, "aa", 20)]
+    assert expected == df3.rows()
 
 
 def test_distinct() -> None:
@@ -190,14 +191,14 @@ def test_distinct() -> None:
 
 def test_frame_sql_globals_error() -> None:
     df1 = pl.DataFrame({"a": [1, 2, 3], "b": [4, 5, 6]})
-    df2 = pl.DataFrame({"a": [2, 3, 4], "b": [7, 6, 5]})  # noqa: F841
+    df2 = pl.DataFrame({"a": [2, 3, 4], "b": [7, 6, 5]})
 
     query = """
         SELECT df1.a, df2.b
         FROM df2 JOIN df1 ON df1.a = df2.a
         ORDER BY b DESC
     """
-    with pytest.raises(SQLInterfaceError, match="relation.*not found.*"):
+    with pytest.raises(SQLInterfaceError, match=r"relation.*not found.*"):
         df1.sql(query=query)
 
     res = pl.sql(query=query, eager=True)
@@ -209,7 +210,7 @@ def test_global_misc_lookup() -> None:
     # as supporting pycapsule (as it can look like it has *any* attr)
     from polars import col  # noqa: F401
 
-    df = pl.DataFrame({"col": [90, 80, 70]})  # noqa: F841
+    df = pl.DataFrame({"col": [90, 80, 70]})
     df_res = pl.sql("SELECT col FROM df WHERE col > 75", eager=True)
     assert df_res.rows() == [(90,), (80,)]
 
@@ -247,12 +248,36 @@ def test_limit_offset() -> None:
         assert len(out) == min(limit, n_values - offset)
 
 
+def test_nested_subquery_table_leakage() -> None:
+    a = pl.LazyFrame({"id": [1, 2, 3]})
+    b = pl.LazyFrame({"val": [2, 3, 4]})
+
+    ctx = pl.SQLContext(a=a, b=b)
+    ctx.execute("""
+      SELECT *
+      FROM a
+      WHERE id IN (
+          SELECT derived.val
+          FROM (SELECT val FROM b) AS derived
+      )
+    """)
+
+    # after execution of the above query, confirm that we don't see the
+    # inner "derived" table alias still being registered in the context
+    with pytest.raises(
+        SQLInterfaceError,
+        match="relation 'derived' was not found",
+    ):
+        ctx.execute("SELECT * FROM derived")
+
+
 def test_register_context() -> None:
-    # use as context manager unregisters tables created within each scope
-    # on exit from that scope; arbitrary levels of nesting are supported.
+    # context manager usage should unregister tables created in each
+    # scope on context exit; supports arbitrary levels of nesting.
     with pl.SQLContext() as ctx:
         _lf1 = pl.LazyFrame({"a": [1, 2, 3], "b": ["m", "n", "o"]})
         _lf2 = pl.LazyFrame({"a": [2, 3, 4], "c": ["p", "q", "r"]})
+
         ctx.register_globals()
         assert ctx.tables() == ["_lf1", "_lf2"]
 
@@ -273,8 +298,8 @@ def test_sql_on_compatible_frame_types() -> None:
     # create various different frame types
     dfp = df.to_pandas()
     dfa = df.to_arrow()
-    dfb = dfa.to_batches()[0]  # noqa: F841
-    dfo = PyCapsuleStreamHolder(df)  # noqa: F841
+    dfb = dfa.to_batches()[0]
+    dfo = PyCapsuleStreamHolder(df)
 
     # run polars sql query against all frame types
     for dfs in (  # noqa: B007
@@ -492,7 +517,9 @@ def test_select_output_heights_20058_21084(filter_expr: str, order_expr: str) ->
 
 
 def test_select_explode_height_filter_order_by() -> None:
-    # Note: `unnest()` from SQL equates to `Expr.explode()`
+    # Note: `unnest()` from SQL equates to `pl.Dataframe.explode()
+    # The ordering is applied after the explosion/unnest.
+    # `
     df = pl.DataFrame(
         {
             "list_long": [[1, 2, 3], [4, 5, 6]],
@@ -502,28 +529,24 @@ def test_select_explode_height_filter_order_by() -> None:
         }
     )
 
-    # Height of unnest is larger than height of sort_key, the sort_key is
-    # extended with NULLs.
-
+    # Unnest/explode is applied at the dataframe level, sort is applied afterward
     assert_frame_equal(
         df.sql("SELECT UNNEST(list_long) as list FROM self ORDER BY sort_key"),
-        pl.Series("list", [2, 1, 3, 4, 5, 6]).to_frame(),
-        check_row_order=False,  # this is wrong at the moment
+        pl.Series("list", [4, 5, 6, 1, 2, 3]).to_frame(),
     )
 
+    # No NULLS: since order is applied after explode on the dataframe level
     assert_frame_equal(
         df.sql(
             "SELECT UNNEST(list_long) as list FROM self ORDER BY sort_key NULLS FIRST"
         ),
-        pl.Series("list", [3, 4, 5, 6, 2, 1]).to_frame(),
-        check_row_order=False,  # this is wrong at the moment
+        pl.Series("list", [4, 5, 6, 1, 2, 3]).to_frame(),
     )
 
     # Literals are broadcasted to output height of UNNEST:
     assert_frame_equal(
         df.sql("SELECT UNNEST(list_long) as list, 1 as x FROM self ORDER BY sort_key"),
-        pl.select(pl.Series("list", [2, 1, 3, 4, 5, 6]), x=1),
-        check_row_order=False,  # this is wrong at the moment
+        pl.select(pl.Series("list", [4, 5, 6, 1, 2, 3]), x=1),
     )
 
     # Note: Filter applies before projections in SQL
@@ -532,15 +555,13 @@ def test_select_explode_height_filter_order_by() -> None:
             "SELECT UNNEST(list_long) as list FROM self WHERE filter_mask ORDER BY sort_key"
         ),
         pl.Series("list", [4, 5, 6]).to_frame(),
-        check_row_order=False,  # this is wrong at the moment
     )
 
     assert_frame_equal(
         df.sql(
             "SELECT UNNEST(list_long) as list FROM self WHERE filter_mask_all_true ORDER BY sort_key"
         ),
-        pl.Series("list", [2, 1, 3, 4, 5, 6]).to_frame(),
-        check_row_order=False,  # this is wrong at the moment
+        pl.Series("list", [4, 5, 6, 1, 2, 3]).to_frame(),
     )
 
 
@@ -575,3 +596,31 @@ def test_count_partition_22665(query: str, result: list[Any]) -> None:
     out = df.sql(query).select("b")
     expected = pl.DataFrame({"b": result}).cast({"b": pl.get_index_type()})
     assert_frame_equal(out, expected)
+
+
+@pytest.mark.parametrize(
+    "query",
+    [
+        # ClickHouse-specific PREWHERE clause
+        "SELECT x, y FROM df PREWHERE z IS NOT NULL",
+        # LATERAL VIEW syntax
+        "SELECT * FROM person LATERAL VIEW EXPLODE(ARRAY(0,125)) tableName AS age",
+        # Oracle-style hierarchical queries
+        """
+        SELECT employee_id, employee_name, manager_id, LEVEL AS hierarchy_level
+        FROM employees
+        START WITH manager_id IS NULL
+        CONNECT BY PRIOR employee_id = manager_id
+        """,
+    ],
+)
+def test_unsupported_select_clauses(query: str) -> None:
+    # ensure we're actively catching unsupported clauses
+    with (
+        pl.SQLContext() as ctx,
+        pytest.raises(
+            SQLInterfaceError,
+            match=r"not.*supported",
+        ),
+    ):
+        ctx.execute(query)
