@@ -153,19 +153,19 @@ def test_join_lazy_on_df() -> None:
 
     with pytest.raises(
         TypeError,
-        match="expected `other` .*to be a 'LazyFrame'.* not 'DataFrame'",
+        match=r"expected `other` .*to be a 'LazyFrame'.* not 'DataFrame'",
     ):
         df_left.lazy().join(df_right, on="Id")  # type: ignore[arg-type]
 
     with pytest.raises(
         TypeError,
-        match="expected `other` .*to be a 'LazyFrame'.* not 'DataFrame'",
+        match=r"expected `other` .*to be a 'LazyFrame'.* not 'DataFrame'",
     ):
         df_left.lazy().join_asof(df_right, on="Id")  # type: ignore[arg-type]
 
     with pytest.raises(
         TypeError,
-        match="expected `other` .*to be a 'LazyFrame'.* not 'pandas.core.frame.DataFrame'",
+        match=r"expected `other` .*to be a 'LazyFrame'.* not 'pandas.core.frame.DataFrame'",
     ):
         df_left.lazy().join_asof(df_right.to_pandas(), on="Id")  # type: ignore[arg-type]
 
@@ -264,7 +264,7 @@ def test_invalid_concat_type_err() -> None:
     )
     with pytest.raises(
         ValueError,
-        match="DataFrame `how` must be one of {'vertical', '.+', 'align_right'}, got 'sausage'",
+        match=r"DataFrame `how` must be one of {'vertical', '.+', 'align_right'}, got 'sausage'",
     ):
         pl.concat([df, df], how="sausage")  # type: ignore[arg-type]
 
@@ -325,7 +325,7 @@ def test_invalid_dtype() -> None:
 
     with pytest.raises(
         TypeError,
-        match="cannot parse input <class 'datetime.tzinfo'> into Polars data type",
+        match=r"cannot parse input <class 'datetime\.tzinfo'> into Polars data type",
     ):
         pl.Series([None], dtype=tzinfo)  # type: ignore[arg-type]
 
@@ -355,7 +355,7 @@ def test_sort_by_different_lengths() -> None:
         }
     )
     with pytest.raises(
-        ComputeError,
+        ShapeError,
         match=r"expressions in 'sort_by' must have matching group lengths",
     ):
         df.group_by("group").agg(
@@ -365,7 +365,7 @@ def test_sort_by_different_lengths() -> None:
         )
 
     with pytest.raises(
-        ComputeError,
+        ShapeError,
         match=r"expressions in 'sort_by' must have matching group lengths",
     ):
         df.group_by("group").agg(
@@ -374,6 +374,16 @@ def test_sort_by_different_lengths() -> None:
             ]
         )
 
+    with pytest.raises(
+        ShapeError,
+        match=r"expressions in 'sort_by' must have matching group lengths",
+    ):
+        df.group_by("group").agg(
+            [
+                pl.col("col1").sort_by(pl.col("col2").first()),
+            ]
+        )
+
 
 def test_err_filter_no_expansion() -> None:
     # df contains floats
@@ -464,7 +474,7 @@ def test_skip_nulls_err() -> None:
     df = pl.DataFrame({"foo": [None, None]})
     with pytest.raises(
         pl.exceptions.InvalidOperationError,
-        match="UDF called without return type, but was not able to infer the output type.",
+        match=r"UDF called without return type, but was not able to infer the output type",
     ):
         df.with_columns(pl.col("foo").map_elements(lambda x: x, skip_nulls=True))
 
@@ -508,7 +518,7 @@ def test_sort_by_err_9259() -> None:
         {"a": [1, 1, 1], "b": [3, 2, 1], "c": [1, 1, 2]},
         schema={"a": pl.Float32, "b": pl.Float32, "c": pl.Float32},
     )
-    with pytest.raises(ComputeError):
+    with pytest.raises(ShapeError):
         df.lazy().group_by("c").agg(
             [pl.col("a").sort_by(pl.col("b").filter(pl.col("b") > 100)).sum()]
         ).collect()
@@ -562,7 +572,7 @@ def test_invalid_is_in_dtypes(
     if expected is None:
         with pytest.raises(
             InvalidOperationError,
-            match="'is_in' cannot check for .*? values in .*? data",
+            match=r"'is_in' cannot check for .*? values in .*? data",
         ):
             df.select(pl.col(colname).is_in(values))
     else:
@@ -580,7 +590,7 @@ def test_sort_by_error() -> None:
     )
 
     with pytest.raises(
-        ComputeError,
+        ShapeError,
         match="expressions in 'sort_by' must have matching group lengths",
     ):
         df.group_by("id", maintain_order=True).agg(
@@ -642,17 +652,10 @@ def test_raise_invalid_arithmetic() -> None:
         df.select(pl.col("a") - pl.col("a"))
 
 
-def test_raise_on_sorted_multi_args() -> None:
-    with pytest.raises(TypeError):
-        pl.DataFrame({"a": [1], "b": [1]}).set_sorted(
-            ["a", "b"]  # type: ignore[arg-type]
-        )
-
-
 def test_err_invalid_comparison() -> None:
     with pytest.raises(
         SchemaError,
-        match="could not evaluate comparison between series 'a' of dtype: date and series 'b' of dtype: bool",
+        match="could not evaluate comparison between series 'a' of dtype: Date and series 'b' of dtype: Boolean",
     ):
         _ = pl.Series("a", [date(2020, 1, 1)]) == pl.Series("b", [True])
 
